import re
import discord
from helpers.redis import RedisHelper


class ActionCore:
    def __init__(self, bot: discord.Client):  # Use discord.Client or commands.Bot for compatibility
        self.redis_helper = RedisHelper()
        self.redis = None
        self.bot = bot

    async def __aenter__(self):
        self.redis = await self.redis_helper.data_connection()
        if not self.redis:
            print("Warning: Failed to establish Redis data connection for ActionCore.")
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.redis_helper.__aexit__(exc_type, exc, tb)
        self.redis = None

    async def setup(self):
        self.redis = await self.redis_helper.data_connection()
        if not self.redis:
            print("Warning: Failed to establish Redis data connection for ActionCore.")

    def guild_by_id(self, guild_id: int) -> discord.Guild | None:
        return self.bot.get_guild(guild_id)

    async def member_by_id(self, guild_id: int, user_id: int) -> discord.Member | None:
        guild = self.guild_by_id(guild_id)
        if guild is None:
            return None

        # Try get from cache
        member = guild.get_member(user_id)
        if member:
            return member

        # Fallback to API lookup
        try:
            return await guild.fetch_member(user_id)
        except discord.NotFound:
            print(f"User {user_id} not found in guild {guild_id}")
        except discord.Forbidden:
            print(f"Missing permissions to fetch member in guild {guild_id}")
        except discord.HTTPException as e:
            print(f"HTTP error while fetching member: {e}")

        return None

    async def channel_by_id(self, guild_id: int, channel_id: int) -> discord.abc.GuildChannel | None:
        guild = self.guild_by_id(guild_id)
        if guild is None:
            return None

        # Try cache
        channel = guild.get_channel(channel_id)
        if channel:
            return channel

        # Fallback to API
        try:
            return await self.bot.fetch_channel(channel_id)
        except discord.NotFound:
            print(f"Channel {channel_id} not found.")
        except discord.Forbidden:
            print("Missing permissions to fetch channel.")
        except discord.HTTPException as e:
            print(f"HTTP error while fetching channel: {e}")

        return None


    def remove_mentions(self, text: str) -> str:
        # Patterns for user, role mentions and @everyone/@here
        mention_patterns = [
            r"<@!?\d+>",      # user mentions, with optional '!' for nickname
            r"<@&\d+>",       # role mentions
            r"@everyone",     # everyone mention
            r"@here"          # here mention
        ]

        pattern = "|".join(mention_patterns)
        cleaned = re.sub(pattern, "", text, flags=re.IGNORECASE)
        return cleaned.strip()
