# Discord libraries
import discord
from discord.ext import commands

# Helpers for NLP intent declaration
from helpers.intent import intent, list_intents
from helpers.intent_cog import IntentCog

# Helper for logging
import uuid
import helpers.logger as log_helper

# Action class to centralize actions the bot can take
from actions.avc import ActionAVC

# Finally, class!
class AutoVoiceChannelCog(IntentCog):
    def __init__(self, bot):
        self.bot = bot
        self.OWNERS = [214577944570494976, 1088598275562668195]
        self.action = ActionAVC(bot)
        self.logger = log_helper.get_logger(__name__)
        self.logger.debug("AutoVoiceChannelCog initialized")

    @commands.Cog.listener()
    async def on_voice_state_update(self, member: discord.Member, before: discord.VoiceState, after: discord.VoiceState):
        """
        Listens for a change in voice state. If a user is joining a new server and that server is considered "monitored",
        we'll create a new VC for that person.
        """
        # Creates UUID to make log help
        CMDID = uuid.uuid1()

        self.logger.debug(f"[{CMDID}] avc on_voice_state_update triggered")
        # Detect if user has left a channel
        if after.channel is None:
            self.logger.debug(f"[{CMDID}] {member.name} disconnected from a channel.")
            self.logger.debug(f"[{CMDID}] Running clean up just to be an over achiever.")
            await self.action.avc_cleanup()

        # Detect if user has joined a new channel
        if after.channel is not None:
            self.logger.debug(f"[{CMDID}] User appears to be present in a channel.")
            if before.channel is not None:
                self.logger.debug(f"[{CMDID}] before was set, so treating this carefully.")
                if after.channel.id != before.channel.id:
                    self.logger.debug(f"[{CMDID}] User appears to have moved VC and not just muted etc.")
                    if await self.action.avc_ismon(after.channel):
                        await self.action.avc_create(member, before, after)

            if before.channel is None:
                self.logger.debug(f"[{CMDID}] User appears to have newly joined a VC")
                if await self.action.avc_ismon(after.channel):
                    await self.action.avc_create(member, before, after)

    @intent([["avc"], ["monitor"]])
    async def avc_monitor_intent(self, ctx, doc):
        """
        Expected phrase: avc monitor <channel_id>
                     or: avc monitor 1234

        Will check to make sure that the person is a guild admin of some sort.
        """
        UUID = log_helper.get_uuid()
        self.logger.debug(f"[{UUID}] avc_monitor_intent called with message: {doc}")

        if not ctx.guild:
            await ctx.reply("This has to be run inside of a guild.")
            return False

        if not ctx.author.guild_permissions.administrator:
            await ctx.reply("You are not an admin in this Discord.")
            return False

        self.logger.debug(f"[{UUID}] checked command is run in a guild and author has admin to the guild.")

        # Extract channel ID
        channel_id = self.action.remove_mentions(doc.text)
        channel_id = int(' '.join(channel_id.split()[2:]))
        self.logger.debug(f"[{UUID}] Extracted {channel_id} as the channel ID")

        # Combine a channel for ZERO reason
        channel = await self.action.channel_by_id(ctx.guild.id, int(channel_id))
        self.logger.debug(f"[{UUID}] Found {channel.name} ({channel.id}) in {ctx.guild.name} ({ctx.guild.id})")

        # Check to see if we're already monitoring it
        if await self.action.avc_ismon(channel):
            await ctx.reply("I'm already monitoring that channel.")
            return False

        if isinstance(channel, discord.VoiceChannel):
            if await self.action.avc_monitor(ctx.guild.id, channel.id):
                await ctx.reply(f"I'm now monitoring {channel.name}")
            else:
                await ctx.reply(f"I could not monitor {channel.name}. Try again later")
        else:
            await ctx.reply(f"{channel.name} is not a voice channel.")




async def setup(bot):
    await bot.add_cog(AutoVoiceChannelCog(bot))

