import os
import redis.asyncio as aioredis  # or 'import aioredis' if using legacy version


class RedisHelper:
    def __init__(self):
        self.REDIS_SERVER  = os.environ.get('REDIS_SERVER', 'localhost')
        self.REDIS_PORT    = int(os.environ.get('REDIS_PORT', '6379'))
        self.REDIS_CONF_DB = int(os.environ.get('REDIS_CONF_DB', '1'))
        self.REDIS_DATA_DB = int(os.environ.get('REDIS_DATA_DB', '0'))

        self._data_connection = None
        self._conf_connection = None

    async def __aenter__(self):
        # Preload both connections for use inside async with block
        await self.data_connection()
        await self.conf_connection()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        if self._data_connection:
            await self._data_connection.close()
            self._data_connection = None

        if self._conf_connection:
            await self._conf_connection.close()
            self._conf_connection = None

    async def data_connection(self):
        if self._data_connection is None:
            try:
                self._data_connection = aioredis.Redis.from_url(
                    f"redis://{self.REDIS_SERVER}:{self.REDIS_PORT}/{self.REDIS_DATA_DB}",
                    decode_responses=True
                )
                await self._data_connection.ping()
                print("Redis data connection established successfully.")
            except aioredis.RedisError as e:
                print(f"Error connecting to Redis data DB: {e}")
                self._data_connection = None
        return self._data_connection

    async def conf_connection(self):
        if self._conf_connection is None:
            try:
                self._conf_connection = aioredis.Redis.from_url(
                    f"redis://{self.REDIS_SERVER}:{self.REDIS_PORT}/{self.REDIS_CONF_DB}",
                    decode_responses=True
                )
                await self._conf_connection.ping()
                print("Redis config connection established successfully.")
            except aioredis.RedisError as e:
                print(f"Error connecting to Redis conf DB: {e}")
                self._conf_connection = None
        return self._conf_connection

